import math
import random
from sklearn.manifold import MDS
import numpy as np
import os
import sys

matrixFilename = "matrix.txt"
prcnt = 0.01 
cliqueOut = "clique.txt"
cliqueSequence = "clique_sequence.txt"

if len(sys.argv) > 1:
    prcnt=float(sys.argv[1])


def d(x,y):
    s = 0
    for i in range(len(x)):
        s += (x[i]-y[i])**2
    return math.sqrt(s)


def teziste(vektori):
    t=[]
    for i in range(len(vektori[0])):
        t.append(0)
    for v in vektori:
        for i in range(len(v)):
            t[i]+=v[i]
    for i in range(len(t)):
        t[i]=t[i]/float(len(vektori))
    return t

def kugla(vektori, teziste, radius):
    rez=[]
    for v in vektori:
        if d(v,teziste)<radius:
            rez.append(v)
    return rez

def kugla_ind(vektori, teziste, radius):
    rez=[]
    for i in range(len(vektori)):
        if d(vektori[i],teziste)<radius:
            rez.append(i)
    return rez


def getBestBall(R, vektori,k,nvn):
    maxx = 0
    maxi = 0
    for i in range(k):
        brtocaka = len(kugla(nvn, nvn[i], 0.5))
        if brtocaka > maxx:
            maxx = brtocaka
            maxi = i
        
    centar = teziste(kugla(nvn, nvn[maxi], 0.5))
    
    tempVektori=vektori[:]
    r = 1
    br = 0
    while r>R:
        tempVektori=kugla(vektori, centar, r)
        r=r*0.99
        br+=1
    return [tempVektori, centar]




def getpsClique(matrix,k): ## pseudo clique

    tmp=[]
    for i in range(k):
        tmp.append(0)

    dm=[] #### distance matrix
    for i in range(k):
        dm.append(tmp[:])

    im=matrix #### incidence matrix


    ### creating distance matrix
    for i in range(k):
        for j in range(k):
            #dm[i][j]=1-im[i][j]
            a=random.random()
            dm[i][j]=math.exp(-10*im[i][j])+a*1/1000.0
            if dm[i][j]<0.51:
                dm[i][j]=0.51+a*1/1000.0

    #### adjusting
    for i in range(k):
        dm[i][i]=0
        im[i][i]=0
    for i in range(k):
        for j in range(i,k):
            dm[j][i]=dm[i][j]


    def d(x,y):
        s = 0
        for i in range(len(x)):
            s += (x[i]-y[i])**2
        return math.sqrt(s)


    embedding=MDS(n_components=5*math.ceil(math.log10(k)),dissimilarity='precomputed')
    nvn=embedding.fit_transform(dm)
    nvnsave = []
    for nvnitem in nvn:
        nvnsave.append(nvnitem.tolist())
    
    
    
    [vec, centar] = getBestBall(0.25, nvn,k,nvn)

    
    bestii=0
    bestr=0
    for ii in range(1100):
        psclique=kugla_ind(nvnsave,centar, ii/1000.0)
    

        seeM = [[0 for _ in range(len(psclique))] for _ in range(len(psclique))] 
        for i in range(len(psclique)):
            for j in range(len(psclique)):
                seeM[i][j]=matrix[psclique[i]][psclique[j]]

        

        cnt=0
        for i in range(len(seeM)):
            for j in range(i+1,len(seeM)):
                if seeM[i][j]==0:
                    cnt=cnt+1

        Nn=(len(seeM)-1)*(len(seeM))/2.0
        if Nn>0:
            if (cnt/Nn)<=prcnt:
                
                bestii=ii
                bestr=ii/1000.0
    
    psclique=kugla_ind(nvnsave,centar, bestr)

    return psclique


   

f=open(matrixFilename,"r")
matrix=f.readlines()
f.close()

dim = len(matrix)
for i in range(dim):
    matrix[i]=matrix[i].rstrip().split()
    for j in range(dim):
        matrix[i][j]=int(matrix[i][j])

printNot01=False
printNotSym=False
for i in range(len(matrix)):
    for j in range(i+1,len(matrix)):
        if matrix[i][j] not in [0,1]:
            printNot01=True
            if matrix[i][j]!=0:
                matrix[i][j]=1
        if matrix[i][j]!=matrix[j][i]:
            printNotSym=True
            matrix[j][i]=matrix[i][j]
            
if printNotSym:
    print("Matrix is not symetric, taking only upper half into calculation...")

if printNot01:
    print("Matrix is not 0-1 matrix, setting all non zero values to one...")
            

printDiag=False
for i in range(len(matrix)):
    if matrix[i][i]!=0:
        printDiag=True
        matrix[i][i]=0

if printDiag:
    print("Incidence matrix is not all zeros on the diagonal... making all zero diagonal...")

psclique = getpsClique(matrix,dim)

f = open(cliqueOut ,"w")
cnt = 0
for i in psclique:
    f.write(str(i)+"\n")
    cnt+=1
f.close()
print("psclique size: " + str(len(psclique)))
print("psclique: %s" % psclique)

##B=[[0 for _ in range(len(psclique))] for _ in range(len(psclique))]
##for i in range(len(psclique)):
##    for j in range(len(psclique)):
##        B[i][j]=matrix[psclique[i]][psclique[j]]
##
##B=np.array(B)
##print(B)




